#pragma once

#include <memory>
#include <string>
#include <vector>

class TemplateNode
{
public:
    TemplateNode(TemplateNode* parent);

    virtual ~TemplateNode() = default;

    TemplateNode* getParent() const;

    virtual void addChild(std::unique_ptr<TemplateNode> child);

    std::size_t getNumChildren() const;

    virtual std::string getIdentifier() const;

    TemplateNode* getChild(std::size_t index) const;

    virtual std::string getRawContent() const;

    template<typename T>
    T* getFirstChildShallow(const std::string& identifier = {}) const
    {
        for (const auto& child : mChildren)
        {
            if (auto ret = dynamic_cast<T*>(child.get()))
            {
                if (!identifier.empty())
                {
                    if (child->getIdentifier() == identifier)
                    {
                        return ret;
                    }
                }
                else
                {
                    return ret;
                }
            }
        }
        return nullptr;
    }

    void setExtensionParent(TemplateNode* parent);

    virtual std::string render(TemplateNode* parentContext = nullptr);

protected:
    std::vector<std::unique_ptr<TemplateNode> > mChildren;
    TemplateNode* mParent{ nullptr };
    TemplateNode* mExtensionParent{ nullptr };
};

using TemplateNodePtr = std::unique_ptr<TemplateNode>;

class TemplateExtends : public TemplateNode
{
public:
    TemplateExtends(TemplateNode* parent, const std::string& path);

    virtual ~TemplateExtends() = default;

    std::string getRawContent() const override;
    std::string getPath() const;

private:
    std::string mPath;
};

class TemplateBlock : public TemplateNode
{
public:
    TemplateBlock(TemplateNode* parent, const std::string& name);

    virtual ~TemplateBlock() = default;

    void addLine(const std::string& line);

    std::string getRawContent() const override;

    std::string renderAsParent(TemplateNode* base);

    std::string render(TemplateNode* parentContext) override;

private:
    std::string mName;
    std::vector<std::string> mBody;
};

class TemplateExpression : public TemplateNode
{
public:
    TemplateExpression(TemplateNode* parent, const std::string& content);

    virtual ~TemplateExpression() = default;

    std::string getRawContent() const override;

private:
    std::string mContent;
};

class TemplateTextBody : public TemplateNode
{
public:
    TemplateTextBody(TemplateNode* parent);

    virtual ~TemplateTextBody() = default;

    void addLine(const std::string& content);

    std::string getRawContent() const override;

    bool hasContent() const;

    std::string render(TemplateNode* parentContext) override;
private:
    std::vector<std::string> mContent;
};
